// Copyright 2016 Canonical Ltd.
// Licensed under the AGPLv3, see LICENCE file for details.

package featuretests

import (
	"github.com/juju/cmd"
	"github.com/juju/cmd/cmdtesting"
	"github.com/juju/loggo"
	jc "github.com/juju/testing/checkers"
	gc "gopkg.in/check.v1"
	"gopkg.in/juju/names.v2"

	apicloud "github.com/juju/juju/api/cloud"
	"github.com/juju/juju/apiserver/params"
	"github.com/juju/juju/cloud"
	"github.com/juju/juju/cmd/juju/commands"
	jujutesting "github.com/juju/juju/juju/testing"
	"github.com/juju/juju/jujuclient"
	"github.com/juju/testing"
)

type CmdCredentialSuite struct {
	jujutesting.JujuConnSuite
}

func (s *CmdCredentialSuite) run(c *gc.C, args ...string) *cmd.Context {
	context := cmdtesting.Context(c)
	command := commands.NewJujuCommand(context)
	c.Assert(cmdtesting.InitCommand(command, args), jc.ErrorIsNil)
	command.Run(context)
	//c.Assert(command.Run(context), jc.ErrorIsNil)
	loggo.RemoveWriter("warning")
	return context
}

func (s *CmdCredentialSuite) TestUpdateCredentialCommand(c *gc.C) {
	//Add dummy cloud to cloud metadata
	s.Home.AddFiles(c, testing.TestFile{
		Name: ".local/share/clouds.yaml",
		Data: `
clouds:
  dummy:
    type: oracle
    description: Dummy Test Cloud Metadata
    auth-types: [ userpass ]
`,
	})

	store := jujuclient.NewFileClientStore()
	store.UpdateCredential("dummy", cloud.CloudCredential{
		AuthCredentials: map[string]cloud.Credential{
			"cred": cloud.NewCredential(cloud.UserPassAuthType, map[string]string{"username": "fred", "password": "secret", "identity-domain": "domain"}),
		},
	})
	s.run(c, "update-credential", "dummy", "cred")

	client := apicloud.NewClient(s.OpenControllerAPI(c))
	defer client.Close()

	tag := names.NewCloudCredentialTag("dummy/admin@local/cred")
	result, err := client.Credentials(tag)
	c.Assert(err, jc.ErrorIsNil)
	c.Assert(result, jc.DeepEquals, []params.CloudCredentialResult{
		{Result: &params.CloudCredential{
			AuthType:   "userpass",
			Attributes: map[string]string{"username": "fred", "identity-domain": "domain"},
			Redacted:   []string{"password"},
		}},
	})
}

func (s *CmdCredentialSuite) TestShowCredentialCommandAll(c *gc.C) {
	ctx := s.run(c, "show-credential")

	c.Assert(cmdtesting.Stderr(ctx), gc.Equals, "")
	c.Assert(cmdtesting.Stdout(ctx), gc.Equals, `
controller-credentials:
  dummy:
    cred:
      content:
        auth-type: userpass
        username: dummy
      models:
        controller: admin
`[1:])
}

func (s *CmdCredentialSuite) TestShowCredentialCommandWithName(c *gc.C) {
	ctx := s.run(c, "show-credential", "dummy", "cred", "--show-secrets")

	c.Assert(cmdtesting.Stderr(ctx), gc.Equals, "")
	c.Assert(cmdtesting.Stdout(ctx), gc.Equals, `
controller-credentials:
  dummy:
    cred:
      content:
        auth-type: userpass
        password: secret
        username: dummy
      models:
        controller: admin
`[1:])
}
